/*
    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "waveformconfigwidget.h"

using namespace SubtitleComposer;

WaveformConfigWidget::WaveformConfigWidget(QWidget *parent)
	: QWidget(parent)
{
	setupUi(this);
}
