/*
    SPDX-FileCopyrightText: 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "scripting_subtitlelinemodule.h"

#include <QDebug>

using namespace SubtitleComposer;

Scripting::SubtitleLineModule::SubtitleLineModule(QObject *parent) :
	QObject(parent)
{}


