/*
    SPDX-FileCopyrightText: 2020-2022 Mladen Milinkovic <max@smoothware.net>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#ifndef SUBTITLETARGET_H
#define SUBTITLETARGET_H

namespace SubtitleComposer {

enum SubtitleTarget {
	Primary = 0,
	Secondary,
	Both,
	SubtitleTargetSize
};

}

#endif // SUBTITLETARGET_H
