/*
    SPDX-FileCopyrightText: 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#ifndef USERACTIONNAMES_H
#define USERACTIONNAMES_H

#define ACT_NEW_SUBTITLE "new_subtitle"
#define ACT_OPEN_SUBTITLE "open_subtitle"
#define ACT_REOPEN_SUBTITLE_AS "reopen_subtitle_as"
#define ACT_RECENT_SUBTITLES "recent_subtitles"
#define ACT_SAVE_SUBTITLE "save_subtitle"
#define ACT_SAVE_SUBTITLE_AS "save_subtitle_as"
#define ACT_CLOSE_SUBTITLE "close_subtitle"
#define ACT_NEW_SUBTITLE_TR "new_subtitle_tr"
#define ACT_OPEN_SUBTITLE_TR "open_subtitle_tr"
#define ACT_REOPEN_SUBTITLE_TR_AS "reopen_subtitle_tr_as"
#define ACT_RECENT_SUBTITLES_TR "recent_subtitles_tr"
#define ACT_SAVE_SUBTITLE_TR "save_subtitle_tr"
#define ACT_SAVE_SUBTITLE_TR_AS "save_subtitle_tr_as"
#define ACT_CLOSE_SUBTITLE_TR "close_subtitle_tr"
#define ACT_UNDO "undo"
#define ACT_REDO "redo"
#define ACT_SPLIT_SUBTITLE "split_subtitle"
#define ACT_JOIN_SUBTITLES "join_subtitles"
#define ACT_INSERT_BEFORE_CURRENT_LINE "insert_before_current_line"
#define ACT_INSERT_AFTER_CURRENT_LINE "insert_after_current_line"
#define ACT_REMOVE_SELECTED_LINES "remove_selected_lines"
#define ACT_SPLIT_SELECTED_LINES "split_selected_lines"
#define ACT_JOIN_SELECTED_LINES "join_selected_lines"
#define ACT_SELECT_ALL_LINES "select_all_lines"
#define ACT_GOTO_LINE "goto_line"
#define ACT_FIND "find"
#define ACT_FIND_NEXT "find_next"
#define ACT_FIND_PREVIOUS "find_previous"
#define ACT_REPLACE "replace"
#define ACT_RETROCEDE_CURRENT_LINE "retrocede_current_line"
#define ACT_ADVANCE_CURRENT_LINE "advance_current_line"
#define ACT_DETECT_ERRORS "detect_errors"
#define ACT_CLEAR_ERRORS "clear_errors"
#define ACT_FIND_ERROR "find_error"
#define ACT_SELECT_NEXT_ERROR "select_next_error"
#define ACT_SELECT_PREVIOUS_ERROR "select_previous_error"
#define ACT_SPELL_CHECK "spell_check"
#define ACT_TOGGLE_SELECTED_LINES_MARK "toggle_selected_lines_mark"
#define ACT_TOGGLE_SELECTED_LINES_BOLD "toggle_selected_lines_bold"
#define ACT_TOGGLE_SELECTED_LINES_ITALIC "toggle_selected_lines_italic"
#define ACT_TOGGLE_SELECTED_LINES_UNDERLINE "toggle_selected_lines_underline"
#define ACT_TOGGLE_SELECTED_LINES_STRIKETHROUGH "toggle_selected_lines_strikethrough"
#define ACT_CHANGE_SELECTED_LINES_TEXT_COLOR "change_selected_lines_text_color"
#define ACT_SHIFT "shift"
#define ACT_SHIFT_SELECTED_LINES_FORWARDS "shift_selected_lines_forwards"
#define ACT_SHIFT_SELECTED_LINES_BACKWARDS "shift_selected_lines_backwards"
#define ACT_ADJUST "adjust"
#define ACT_CHANGE_FRAME_RATE "change_frame_rate"
#define ACT_DURATION_LIMITS "duration_limits"
#define ACT_AUTOMATIC_DURATIONS "automatic_durations"
#define ACT_MAXIMIZE_DURATIONS "maximize_durations"
#define ACT_FIX_OVERLAPPING_LINES "fix_overlapping_lines"
#define ACT_SYNC_WITH_SUBTITLE "sync_with_subtitle"
#define ACT_ADJUST_TEXTS "adjust_texts"
#define ACT_UNBREAK_TEXTS "unbreak_texts"
#define ACT_SIMPLIFY_SPACES "simplify_spaces"
#define ACT_CHANGE_CASE "change_case"
#define ACT_FIX_PUNCTUATION "fix_punctuation"
#define ACT_TRANSLATE "translate"
#define ACT_EDIT_CURRENT_LINE_IN_PLACE "edit_current_line_in_place"
#define ACT_OPEN_VIDEO "open_video"
#define ACT_RECENT_VIDEOS "recent_videos"
#define ACT_CLOSE_VIDEO "close_video"
#define ACT_TOGGLE_FULL_SCREEN "toggle_full_screen"
#define ACT_STOP "stop"
#define ACT_PLAY_PAUSE "play_pause"
#define ACT_SEEK_BACKWARD "seek_backward"
#define ACT_SEEK_FORWARD "seek_forward"
#define ACT_SEEK_TO_CURRENT_LINE "seek_to_current_line"
#define ACT_SEEK_TO_NEXT_LINE "seek_to_next_line"
#define ACT_SEEK_TO_PREVIOUS_LINE "seek_to_previous_line"
#define ACT_PLAY_CURRENT_LINE_AND_PAUSE "play_current_line_and_pause"
#define ACT_SET_CURRENT_LINE_SHOW_TIME "set_current_line_show_time"
#define ACT_SET_CURRENT_LINE_HIDE_TIME "set_current_line_hide_time"
#define ACT_SHIFT_TO_VIDEO_POSITION "shift_to_video_position"
#define ACT_CURRENT_LINE_FOLLOWS_VIDEO "current_line_follows_video"
#define ACT_TOGGLE_MUTED "toggle_muted"
#define ACT_INCREASE_VOLUME "increase_volume"
#define ACT_DECREASE_VOLUME "decrease_volume"
#define ACT_DEMUX_TEXT_STREAM "demux_text_stream"
#define ACT_SET_ACTIVE_AUDIO_STREAM "set_active_audio_stream"
#define ACT_INCREASE_SUBTITLE_FONT "increase_font_size"
#define ACT_DECREASE_SUBTITLE_FONT "decrease_font_size"
#define ACT_SET_ACTIVE_SUBTITLE_STREAM "set_active_subtitle_stream"
#define ACT_ANCHOR_TOGGLE "anchor_toggle"
#define ACT_ANCHOR_REMOVE_ALL "anchor_remove_all"
#define ACT_SCRIPTS_MANAGER "scripts_manager"
#define ACT_WAVEFORM_ZOOM_IN "waveform_zoom_in"
#define ACT_WAVEFORM_ZOOM_OUT "waveform_zoom_out"
#define ACT_WAVEFORM_AUTOSCROLL "waveform_autoscroll"
#define ACT_ASR_IMPORT_AUDIO_STREAM "asr_import_audio_stream"
#define ACT_PLAY_RATE_INCREASE "playrate_increase"
#define ACT_PLAY_RATE_DECREASE "playrate_decrease"
#define ACT_STEP_FRAME_BACKWARD "frame_step_backward"
#define ACT_STEP_FRAME_FORWARD "frame_step_forward"
#endif
