/*
    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "speechplugin.h"

using namespace SubtitleComposer;

SpeechPlugin::SpeechPlugin()
	: QObject(nullptr)
{

}
